@@ -5,7 +5,7 @@ use strict;
 use warnings;
 no if $] >= 5.017011, warnings => 'experimental::smartmatch';
 
-our $VERSION = 0.09;
+our $VERSION = 0.10;
 
 my$self;
 
@@ -18,7 +18,7 @@ sub new{
 	my $class = ref($type)||$type;
 	$self = {
 		totalTransactions => undef,
-		minSupport => 0,
+		minSupport => 1,
 		minConfidence => undef,
 		minLift => undef,
 		minLeverage => undef,
@@ -28,13 +28,11 @@ sub new{
 		messages => undef,
 		itemsKeyDescription => undef,
 		itemsKeyTransactions => undef,
-		quantityPossibleRules => undef,
 		limitRules => undef,
 		frequentItemset => [],
-		recursively => 0,
 		associationRules => undef,
 		implications => undef,
-		itemsetSize => 2,
+		largeItemsetSize => 2,
 		rule => 1
 	};
 	bless($self,$class);
@@ -42,61 +40,59 @@ sub new{
 }
 
 sub validate_data{
-	(defined $self->{totalTransactions})
-		or die("Error: \$apriori->{totalTransactions} is not defined!\n");
 	(defined $self->{itemsKeyDescription})
 		or die("Error: \$apriori->{itemsKeyDescription} is not defined!\n");
 	(defined $self->{itemsKeyTransactions})
 		or die("Error: \$apriori->{itemsKeyTransactions} is not defined!\n");
 }
 
-sub generate_rules{
+sub quantity_possible_rules{
 	$self->validate_data;
-	my @candidateItemsetSizeOne = $self->data_persistence;
-	@candidateItemsetSizeOne = grep{((scalar(@{$self->{itemsKeyTransactions}{$_}})/$self->{totalTransactions})*100)>=$self->{minSupport}}@candidateItemsetSizeOne;
-	if($self->{messages}&&!$self->{recursively}){
-		my $quantityItems = scalar(@candidateItemsetSizeOne);
-		$self->{quantityPossibleRules} = ((3**$quantityItems)-(2**($quantityItems+1))+1);
-		print "\n$quantityItems items, $self->{quantityPossibleRules} possible rules";
-	}
-	$self->{recursively} = 1;
-	$self->association_rules_itemset_size(\@candidateItemsetSizeOne);
+	my $quantityItems = scalar(keys(%{$self->{itemsKeyDescription}}));
+	return ((3**$quantityItems)-(2**($quantityItems+1))+1);
 }
 
-sub association_rules_itemset_size{
-	my @candidateItemsetSizeOne;
-	if($self->{recursively}){
-		@candidateItemsetSizeOne = @{$_[1]};
-	}
-	else{
-		$self->validate_data;
-		(scalar(@_)==2)
-			or die("Error: Invalid number of arguments! It is expected a number representing the size of itemset!\n");
-		@candidateItemsetSizeOne = $self->data_persistence;
-		((scalar(@candidateItemsetSizeOne)>=$_[1])&&($_[1]>=2))
-			or die("Error: Quantity of items should be greater or equal the size of itemset and size of the itemset greater or equal to 2!\n");
-		@candidateItemsetSizeOne = grep{((scalar(@{$self->{itemsKeyTransactions}{$_}})/$self->{totalTransactions})*100)>=$self->{minSupport}}@candidateItemsetSizeOne;
-		$self->{itemsetSize} = $_[1];
-	}
-	my @newCandidateItemsetSizeOne;
+sub generate_rules{
+	$self->validate_data;
+	my @largeItemsetSizeOne = $self->data_persistence;
 	if($self->{messages}){
-		print "\nItemset size ", $self->{itemsetSize}, ", ", scalar(@candidateItemsetSizeOne), " items ";
+		my $quantityItems = scalar(keys(%{$self->{itemsKeyDescription}}));
+		print "\n$quantityItems items, ${\$self->quantity_possible_rules} possible rules";
+	}
+	my @transactions;
+	foreach my$item(keys(%{$self->{itemsKeyDescription}})){
+		foreach my$transaction(@{$self->{itemsKeyTransactions}{$item}}){
+			if(!($transaction~~@transactions)){
+				push@transactions,$transaction;
+			}
+		}
+	}
+	$self->{totalTransactions} = scalar(@transactions);
+	my @frequentItemset = grep{((scalar(@{$self->{itemsKeyTransactions}{$_}})/$self->{totalTransactions})*100)>=$self->{minSupport}}@largeItemsetSizeOne;
+	$self->association_rules(\@frequentItemset);
+}
+
+sub association_rules{
+	my @largeItemsetSizeOne = @{$_[1]};
+	my @frequentItemset;
+	if($self->{messages}){
+		print "\nLarge itemset size ${\$self->{largeItemsetSize}}, ${\scalar(@largeItemsetSizeOne)} items ";
 		print "\nProcessing... ";
 	}
-	for(my$itemsetSize=2;$itemsetSize<=$self->{itemsetSize};$itemsetSize++){
+	for(my$largeItemsetSize=2;$largeItemsetSize<=$self->{largeItemsetSize};$largeItemsetSize++){
 		my $code=<<'CODE';
 my @candidateItemset;
 $self->{implications}={};
-for(my$k=0;$k<=$#candidateItemsetSizeOne;$k++){
-	push @candidateItemset,$candidateItemsetSizeOne[$k];
+for(my$k=0;$k<=$#largeItemsetSizeOne;$k++){
+	push @candidateItemset,$largeItemsetSizeOne[$k];
 CODE
-		for(my$i=2;$i<=$itemsetSize;$i++){
+		for(my$i=2;$i<=$largeItemsetSize;$i++){
 			$code.=<<'CODE';
-for(my$l=0;$l<=$#candidateItemsetSizeOne;$l++){
-	if($candidateItemsetSizeOne[$l] ~~ @candidateItemset){
+for(my$l=0;$l<=$#largeItemsetSizeOne;$l++){
+	if($largeItemsetSizeOne[$l] ~~ @candidateItemset){
 		next;
 	}
-	push @candidateItemset,$candidateItemsetSizeOne[$l];
+	push @candidateItemset,$largeItemsetSizeOne[$l];
 CODE
 		}
 		$code.=<<'CODE';
@@ -136,15 +132,15 @@ for(my$antecedentSize=0;$antecedentSize<$#candidateItemset;$antecedentSize++){
 	next if(defined $self->{minCoverage} && $coverage < $self->{minCoverage});
 	$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{rule} = ["{ @antecedent } => { @consequent }", $support, $confidence, $lift, $leverage, $conviction, $coverage];
 	$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{items} = [@antecedent, @consequent];
-	my@items=grep{!($_~~@newCandidateItemsetSizeOne)}@antecedent;
-	push @newCandidateItemsetSizeOne,@items;
-	@items=grep{!($_~~@newCandidateItemsetSizeOne)}@consequent;
-	push @newCandidateItemsetSizeOne,@items;
-	return if defined $self->{limitRules} && $self->{rule} == $self->{limitRules};
+	my@items=grep{!($_~~@frequentItemset)}@antecedent;
+	push @frequentItemset,@items;
+	@items=grep{!($_~~@frequentItemset)}@consequent;
+	push @frequentItemset,@items;
+	return if(defined $self->{limitRules} && $self->{rule} == $self->{limitRules});
 	$self->{rule}++;
 }
 CODE
-		for(my$j=2;$j<=$itemsetSize;$j++){
+		for(my$j=2;$j<=$largeItemsetSize;$j++){
 			$code.=<<'CODE';
 	pop @candidateItemset;
 }
@@ -154,22 +150,23 @@ CODE
 	pop @candidateItemset;
 }
 CODE
-		if($itemsetSize==$self->{itemsetSize}){
+		if($largeItemsetSize==$self->{largeItemsetSize}){
 			eval $code;
 			die("Error: $@\n") if $@;
 			if($self->{messages}){
-				print "\nFrequent itemset: { @newCandidateItemsetSizeOne }, ", scalar(@newCandidateItemsetSizeOne), " items ";
+				print "\nFrequent itemset: { @frequentItemset }, ${\scalar(@frequentItemset)} items ";
 			}
-			if($self->{associationRules}){
-				@{$self->{frequentItemset}}=@newCandidateItemsetSizeOne;
+			if(defined $self->{associationRules}){
+				@{$self->{frequentItemset}}=@frequentItemset;
 				$self->output;
 			}
 		}
 	}
-	if((scalar(@newCandidateItemsetSizeOne)>=($self->{itemsetSize}+1))&&$self->{recursively}){
-		$self->{itemsetSize}++;
+	return if(defined $self->{limitRules} && $self->{rule} == $self->{limitRules});
+	if(scalar(@frequentItemset)>=($self->{largeItemsetSize}+1)){
+		$self->{largeItemsetSize}++;
 		$self->{associationRules} = undef;
-		$self->association_rules_itemset_size(\@newCandidateItemsetSizeOne);
+		$self->association_rules(\@frequentItemset);
 	}
 }
 
@@ -238,9 +235,9 @@ sub data_persistence{
 
 sub file{
 	if($self->{messages}){
-		print "\nExporting to file \"output_itemset_size_$self->{itemsetSize}.txt\"... ";
+		print "\nExporting to file \"output_large_itemset_size_$self->{largeItemsetSize}.txt\"... ";
 	}
-	open(FILE,">output_itemset_size_$self->{itemsetSize}.txt");
+	open(FILE,">output_large_itemset_size_$self->{largeItemsetSize}.txt");
 	print FILE "Rules\tSupport %";
 	print FILE "\tConfidence %" if defined $self->{minConfidence};
 	print FILE "\tLift" if defined $self->{minLift};
@@ -284,10 +281,10 @@ sub file{
 sub excel{
 	require Excel::Writer::XLSX;
 	if($self->{messages}){
-		print "\nExporting to excel \"output_itemset_size_$self->{itemsetSize}.xlsx\"... ";
+		print "\nExporting to excel \"output_large_itemset_size_$self->{largeItemsetSize}.xlsx\"... ";
 	}
-	my $workbook  = Excel::Writer::XLSX->new("output_itemset_size_$self->{itemsetSize}.xlsx") 
-		or die("\nError: \"output_itemset_size_$self->{itemsetSize}.xlsx\" $!");
+	my $workbook  = Excel::Writer::XLSX->new("output_large_itemset_size_$self->{largeItemsetSize}.xlsx") 
+		or die("\nError: \"output_large_itemset_size_$self->{largeItemsetSize}.xlsx\" $!");
 	my $worksheet = $workbook->add_worksheet();
 	my $bold = $workbook->add_format(bold => 1);
 	my $headings = ['Rules', 'Support %'];
@@ -295,7 +292,7 @@ sub excel{
 	push @{$headings},'Lift' if defined $self->{minLift};
 	push @{$headings},'Leverage' if defined $self->{minLeverage};
 	push @{$headings},'Conviction' if defined $self->{minConviction};
-	push @{$headings},'Coverage' if defined $self->{minCoverage};
+	push @{$headings},'Coverage %' if defined $self->{minCoverage};
 	my(@rules,@support,@confidence,@lift,@leverage,@conviction,@coverage);	
 	foreach my$rule(sort{$a<=>$b}keys(%{$self->{associationRules}})){
 		push @rules,"R$rule";
@@ -387,7 +384,7 @@ sub excel{
 			$line++;
 		}
 		if(defined $self->{minCoverage}){
-			$worksheet->write("A$line","Coverage: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[6]");
+			$worksheet->write("A$line","Coverage: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[6] %");
 			$line++;
 		}
 		$worksheet->write("A$line","Items:");
@@ -435,13 +432,11 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	my $apriori = new Data::Mining::Apriori;
 
-	$apriori->{totalTransactions}=10; # The total number of transactions
-
-	$apriori->{minSupport}=1.55; # The minimum support(percent)
+	$apriori->{minSupport}=1.55; # The minimum support, default values is 1(percent)
 
 	$apriori->{minConfidence}=1.55; # The minimum confidence(percent, optional)
 
-	$apriori->{minLift}=0; # The minimum lift(optional)
+	$apriori->{minLift}=1; # The minimum lift(optional)
 
 	$apriori->{minLeverage}=0; # The minimum leverage(optional)
 
@@ -449,8 +444,6 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	$apriori->{minCoverage}=0; # The minimum coverage(percent, optional)
 
-	$apriori->{limitRules}=10; # The limit of rules(optional)
-
 	$apriori->{output}=1; # The output type (1 - Export to text file delimited by tab; 2 - Export to excel file with chart)(optional)
 
 	$apriori->{messages}=1; # A value boolean to display the messages(optional)
@@ -464,24 +457,27 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 	@{$apriori->{itemsKeyTransactions}{102}}=('1102',1103,'1104',1105,'1106',1107,1108,'1109',1110);
 	@{$apriori->{itemsKeyTransactions}{'103'}}=('1101',1102,1103,'1104','1106',1107,1108,'1109',1110);
 
-	$apriori->generate_rules; # Generate association rules to no longer meet the minimum support and confidence
-	# or
-	#$apriori->association_rules_itemset_size(3); # Generate rules from a set of items size 3, for example
+	print "\n${\$apriori->quantity_possible_rules}"; # Prints the quantity of possible rules
+
+	$apriori->{limitRules}=10; # The limit of rules
+
+	$apriori->generate_rules; # Generate association rules to no longer meet the minimum support, confidence, lift, leverage, conviction, coverage or limit of rules
 
 	print "\n@{$apriori->{frequentItemset}}\n"; # Show frequent items
 
 	#output messages
 
+	12
 	3 items, 12 possible rules
-	Itemset size 2, 3 items 
-	Processing... 
-	Frequent itemset: { 101, 103, 102 }, 3 items 
-	Exporting to file "output_itemset_size_2.txt"... 
-	Itemset size 3, 3 items 
-	Processing... 
-	Frequent itemset: { 101, 102, 103 }, 3 items 
-	Exporting to file "output_itemset_size_3.txt"... 
-	101, 102, 103
+	Large itemset size 2, 3 items
+	Processing...
+	Frequent itemset: { 103, 102, 101 }, 3 items
+	Exporting to excel "output_large_itemset_size_2.xlsx"...
+	Large itemset size 3, 3 items
+	Processing...
+	Frequent itemset: { 103, 101, 102 }, 3 items
+	Exporting to excel "output_large_itemset_size_3.xlsx"...
+	103, 101, 102
 
 	#output file "output_itemset_size_2.txt"
 
@@ -611,13 +607,11 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	$apriori = new Data::Mining::Apriori;
 
-	$apriori->{totalTransactions}=$totalTransactions;
-
 	$apriori->{minSupport}=1.55;
 
 	$apriori->{minConfidence}=1.55;
 
-	$apriori->{minLift}=0;
+	$apriori->{minLift}=1;
 
 	$apriori->{minLeverage}=0;
 
@@ -625,8 +619,6 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	$apriori->{minCoverage}=0;
 
-	$apriori->{limitRules}=10;
-
 	$apriori->{output}=1;
 
 	$apriori->{messages}=1;
@@ -661,9 +653,11 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 		}
 	}
 
+	print "\n${\$apriori->quantity_possible_rules}";
+
+	$apriori->{limitRules}=10;
+
 	$apriori->generate_rules;
-	# or
-	#$apriori->association_rules_itemset_size(3);
 
 	print "\n@{$apriori->{frequentItemset}}\n";
 
@@ -679,35 +673,35 @@ The total number of transactions.
 
 =head2 minSupport
 
-The minimum support(percent).
+The minimum support.(percent)
 
 =head2 minConfidence
 
-The minimum confidence(percent, optional).
+The minimum confidence.(percent, optional)
 
 =head2 minLift
 
-The minimum lift(optional).
+The minimum lift.(optional)
 
 =head2 minLeverage
 
-The minimum leverage(optional).
+The minimum leverage.(optional)
 
 =head2 minConviction
 
-The minimum conviction(optional).
+The minimum conviction.(optional)
 
 =head2 minCoverage
 
-The minimum coverage(percent, optional).
+The minimum coverage.(percent, optional)
 
 =head2 limitRules
 
-The limit of rules(optional).
+The limit of rules.(optional)
 
 =head2 output
 
-The output type (optional):
+The output type:(optional)
 
 =over 4
 
@@ -719,7 +713,7 @@ The output type (optional):
 
 =head2 messages
 
-A value boolean to display the messages(optional).
+A value boolean to display the messages.(optional)
 
 =head2 itemsKeyDescription
 
@@ -729,18 +723,10 @@ Hash table to add items by key and description.
 
 Reference to array, to add the transactions of each item per key.
 
-=head2 quantityPossibleRules
-
-Quantity of possible rules.
-
 =head2 frequentItemset
 
 Frequent itemset.
 
-=head2 recursively
-
-A value boolean to generate association rules until no set of items meets the minimum support or minimum confidence.
-
 =head2 associationRules
 
 A data structure to store the name of the rule, key items, implication, support, confidence, lift, leverage, conviction and coverage.
@@ -771,21 +757,17 @@ A data structure to store the name of the rule, key items, implication, support,
 
 Creates a new instance of Data::Mining::Apriori.
 
+=head2 quantity_possible_rules
+
+Returns the quantity of possible rules.
+
 =head2 generate_rules
 
-Generate association rules until no set of items meets the minimum support, confidence, lift, leverage, conviction and coverage.
+Generate association rules until no set of items meets the minimum support, confidence, lift, leverage, conviction, coverage or limit of rules.
 
-=head2 association_rules_itemset_size
+=head2 association_rules
 
-Generates association rules by size set of items. Accepts the following argument:
-
-=over 4
-
-=item * An integer.
-
-Representing the size of the set of items.
-
-=back
+Generates association rules by size set of larger itemsets.
 
 =head1 AUTHOR
 
